/*
 * Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
 
package hotlava.proxy;

import java.io.*;
import java.net.*;
import java.util.Properties;

/**
 * The Cookie Cutter
 *
 * TODO: Add regular expressions to the site names
 *
 * <p>Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
public class CookieCutter extends Proxylet
{
	protected PrintStream debug;

	protected boolean policy;
	protected Properties allowSites;
	protected Properties denySites;

	public void debug(PrintStream out) { debug = out; }

	public CookieCutter()
	{
		policy = true;
		allowSites = new Properties();
		denySites = new Properties();
	}

	public CookieCutter(Proxylet p)
	{
		this();
		setNext(p);
	}

    public void
	init(Properties p)
		throws ProxyletException
	{
		String policyStr = p.getProperty("hotlava.proxy.CookieCutter.defaultPolicy", "allow");
		String allowSitesStr = p.getProperty("hotlava.proxy.CookieCutter.allowSites", "");
		String denySitesStr = p.getProperty("hotlava.proxy.CookieCutter.denySites", "");

		if (policyStr.toLowerCase().equals("allow"))
			policy = true;
		else if (policyStr.toLowerCase().equals("deny"))
			policy = false;
		else
			throw new ProxyletException("hotlava.proxy.CookieCutter.defaultPolicy contains invalid value");


		String denyStr = "deny";
		String allowStr = "allow";
		int pos = 0;
		int oldpos = 0;
		if (allowSitesStr.length() > 0)
		{
			while ((pos = allowSitesStr.indexOf(',', oldpos)) != -1)
			{
				allowSites.put(allowSitesStr.substring(oldpos, pos), allowStr);
				oldpos = pos+1;
			}
			allowSites.put(allowSitesStr.substring(oldpos), allowStr);
		}

		pos = 0;
		oldpos = 0;
		if (denySitesStr.length() > 0)
		{
			while ((pos = denySitesStr.indexOf(',', oldpos)) != -1)
			{
				denySites.put(denySitesStr.substring(oldpos, pos), denyStr);
				oldpos = pos+1;
			}
			denySites.put(denySitesStr.substring(oldpos), denyStr);
		}
	}

	public void
	handleRequest(hotlava.net.http.Request r, DataOutput os)
		throws IOException, ProxyletException
	{
		boolean pass = policy;

		if (allowSites.getProperty(r.host()) != null)
			pass = true;
		if (denySites.getProperty(r.host()) != null)
			pass = false;

		if (pass == false)
		{
			if (debug != null)
			{
				debug.println("Removing cookie: "+r.getHeader("cookie"));
			}
			r.removeHeader("cookie");
		}

		next.handleRequest(r, os);
	}
}
