/*
 * Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */

package hotlava.proxy;

import java.io.*;
import java.util.*;
import hotlava.util.IniFile;
import hotlava.util.IniFileFormatException;

/**
 * @author Hotlava Ltd
 * @version $Revision: 1.1 $
 * @version $State: Exp $
 *
 * <p>Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
public class Proxy
	extends Thread
{
	protected String propertyPrefix = "hotlava.proxy.";
	protected hotlava.net.http.Server server;
	protected Vector proxylets = new Vector();

	/**
	 * Set the debug PrintStream
	 * @param out the PrintStream where debugging is sent, unless null
	 */
	public void debug(PrintStream out) { server.debug(out); }


	public
	Proxy()
		throws IOException, ProxyException
	{
		init();
	}

	/**
	 * Create a Proxy object from an rc file
	 * The rc file is parsed, and the values placed into the
	 * environment.  Then the Proxy object is created using
	 * the environment.
	 */
	public
	Proxy(File rcFile)
		throws IOException, IniFileFormatException, ProxyException
	{
		FileInputStream fis = new FileInputStream(rcFile);
		IniFile ini = new IniFile(fis);

		//
		// Iterate through ini fields, and add to system properties
		//
		Properties props = System.getProperties();
		for (Enumeration sections = ini.sections(); sections.hasMoreElements() ;)
		{
			String section = (String)sections.nextElement();
			String prefix = propertyPrefix + section + ".";

			for (Enumeration items = ini.items(section); items.hasMoreElements() ;)
			{
				String item = (String)items.nextElement();
				String field = prefix + item;
				
				Object old = props.put(field, ini.get(section, item));
				if (old != null) // Don't override existing properties
					props.put(field, old);
			}
		}

		init();
	}

	public void
	init()
		throws IOException, ProxyException
	{
		Properties props = System.getProperties();

		//
		// Read in the list of proxylets
		//
		String proxyletsStr = props.getProperty(propertyPrefix+"main.proxylets", "UserAgent");
		int pos = 0;
		int oldpos = 0;
		if (proxyletsStr.length() > 0)
		{
			while ((pos = proxyletsStr.indexOf(',', oldpos)) != -1)
			{
				proxylets.addElement(proxyletsStr.substring(oldpos, pos));
				oldpos = pos+1;
			}
			proxylets.addElement(proxyletsStr.substring(oldpos));
		}


		Proxylet proxylet;
		Proxylet chain = null;
		for (int i = proxylets.size()-1; i >= 0; i--)
		{
			String proxyletStr = (String)proxylets.elementAt(i);
			String classname = props.getProperty(propertyPrefix+proxyletStr+".classname");
			if (classname == null)
				throw new ProxyException("Classname not defined for proxylet "+proxyletStr);
			try {
				proxylet = (Proxylet)Class.forName(classname).newInstance();
				proxylet.setNext(chain);
				proxylet.init(props);
				chain = proxylet;
			}
			catch(ClassCastException e) {
				throw new ProxyException("Unable to load proxylet "+classname+" (not derived from Proxylet)");
			}
			catch(InstantiationException e) {
				throw new ProxyException("Unable to instantiate proxylet "+classname);
			}
			catch(IllegalAccessException e) {
				throw new ProxyException("Access to proxylet "+classname+" not permitted");
			}
			catch(ClassNotFoundException e) {
				throw new ProxyException("Cannot find proxylet "+classname);
			}
			catch(ProxyletException e) {
				throw new ProxyException("Problems initialising "+classname+" proxylet: "+e.getMessage());
			}
		}

		String portStr = props.getProperty(propertyPrefix+"main.port", "8080");
		int port;
		try {
			port = Integer.parseInt(portStr);
		}
		catch (NumberFormatException nfe) {
			throw new ProxyException("[main] port field not numeric");
		}

		server = new hotlava.net.http.Server(port);

		String clearInputStr = props.getProperty(propertyPrefix+"main.clearInput", "true").toLowerCase();
		if (clearInputStr.equals("true"))
			server.clearInput(true);
		else if (clearInputStr.equals("false"))
			server.clearInput(false);
		else
			throw new ProxyException("[main] clearInput field not a boolean value");

		server.handler(chain);
	}


	/**
	 * Start serving
	 */
	public void
	run()
	{
		try {
			server.serve();
		}
		catch(IOException e) {
//			ui.reportException(e);
		}
	}


	/**
	 * Used to run this as a standalone proxy application
	 */
	public static void
	main(String[] argv)
		throws IOException, IniFileFormatException, ProxyException
	{
		boolean magnus = true;
		File rcFile = null;
		String dir = System.getProperty("user.home");
		if (magnus == true)
			dir = "/users/server/students/mkjohans";
		if (dir == null)
			rcFile = new File(".proxyrc");
		else
			rcFile = new File(dir, ".proxyrc");

		try {
			Proxy main = new Proxy(rcFile);
			main.start();
			main.join();
		}
		catch(IniFileFormatException e) {
			System.out.println(e.getMessage());
			System.exit(-1);
		}
		catch(ProxyException e) {
			System.out.println(e.getMessage());
			System.exit(-1);
		}
		catch(InterruptedException e) {
		}
	}
}
