/*
 * Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
 
package hotlava.proxy;

import java.io.*;
import java.util.Properties;

/**
 * The abstract base class for all 'proxylets'
 *
 * @author Hotlava Ltd
 * @version $Revision: 1.1 $
 * @version $State: Exp $
 *
 * <p>Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
public abstract class Proxylet
	implements hotlava.net.http.Handler
{
	protected Proxylet next;

	/**
	 * Both protected and abstract, so this class must be derived from to be useful.
	 */
	protected Proxylet()
	{
		next = null;
	}

	/**
	 * Both protected and abstract, so this class must be derived from to be useful.
	 */
	protected Proxylet(Proxylet p)
	{
		next = p;
	}

	/**
	 * Set the next proxylet.  Used in constructing a proxylet 'chain'.
	 * @param p the next proxylet
	 */
	public void
	setNext(Proxylet p)
	{
		next = p;
	}

	/**
	 * Initialise the proxylet from a set of properties
	 * Usually called by hotlava.proxy.Proxy, during the
	 * ini-file configuration process.
	 * @param p a set of properties
	 * @see Proxy
	 */
	public abstract void init(Properties p) throws ProxyletException;

	/**
	 * This method is overriden in order to handle http requests
	 * @param r the http request to be processed
	 * @param os the OutputStream to where the data should be sent
	 */
	public abstract void handleRequest(hotlava.net.http.Request r, DataOutput os)
		throws IOException, ProxyletException;
}
