/*
 * Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
 
package hotlava.proxy;

import java.io.*;
import java.util.Properties;

/**
 * The UserAgent proxylet - passes requests to the 'net
 *
 * TODO - This class needs some work to keep connections alive,
 * in order to improve performance
 * (Actually that should be in hotlava.net.http.ProxyHandler)
 *
 * <p>Copyright (C) 1997 Gary Howland (gary@hotlava.com)
 * All rights reserved.
 */
public class UserAgent
	extends Proxylet
{
	private hotlava.net.http.ProxyHandler proxy;

	/**
	 * Set the debug PrintStream
	 * @param out the PrintStream where debugging is sent, unless null
	 */
	public void debug(PrintStream out) { proxy.debug(out); }

	/**
	 * Set the host used for proxying requests
	 * @param host The proxy host, or null if no proxy required
	 */
	public void host(String host) { proxy.host(host); }

	/**
	 * Set the port used for proxying requests
	 * @param port The proxy port
	 */
	public void port(int port) { proxy.port(port); }


	/**
	 * Creates a UserAgent object
	 */
	public UserAgent()
	{
		proxy = new hotlava.net.http.ProxyHandler();
	}

	/**
	 * Creates a UserAgent object
	 */
	public UserAgent(Proxylet p)
	{
		this();
		setNext(p);
	}

	/**
	 * Creates a UserAgent object, configured to use a proxy.
	 * @param proxy_host the proxy host
	 * @param proxy_port the proxy port
	 */
	public
	UserAgent(Proxylet p, String proxy_host, int proxy_port)
	{
		this(p);
		host(proxy_host);
		port(proxy_port);
	}

	/**
	 * Initialise an object from a set of properties
	 * @param p a set of properties
	 */
	public void
	init(Properties p)
		throws ProxyletException
	{
		//
		// First get the proxy settings out of the way
		//
		String proxySetStr = p.getProperty("hotlava.proxy.UserAgent.proxySet", "false");
		String proxyHostStr = p.getProperty("hotlava.proxy.UserAgent.proxyHost", "localhost");
		String proxyPortStr = p.getProperty("hotlava.proxy.UserAgent.proxyPort", "8080");

		//
		// Now allow the environment to override the proxy settings
		// Is this sensible behaviour?
		//
		String s;
		if ((s = System.getProperty("proxySet")) != null)
			proxySetStr = s;
		if ((s = System.getProperty("proxyHost")) != null)
			proxyHostStr = s;
		if ((s = System.getProperty("proxyPort")) != null)
			proxyPortStr = s;

		if (proxySetStr.toLowerCase().equals("true"))
		{
			host(proxyHostStr);
			try {
				port(Integer.parseInt(proxyPortStr));
			}
			catch (NumberFormatException nfe) {
				throw new ProxyletException("Port field not numeric");
			}
		}
	}

	/**
	 * Handle a request, as required by the Proxylet base class
	 * @param r the http request to be handled
	 * @param os the DataOutput stream to where the request reply should be sent
	 */
	public void
	handleRequest(hotlava.net.http.Request r, DataOutput os)
		throws IOException, ProxyletException
	{
		try {
			proxy.handleRequest(r, os);
		}
		catch (hotlava.net.http.HandlerException e) {
			throw new ProxyletException(e.getMessage());
		}
	}
}
